@@ -38,6 +38,7 @@ class Request(object):
     def __init__(self, socket, client_address, server_address, debug=False):
         self.debug = debug
         self.socket = socket
+    
         self.client_address = client_address
         self.server_address = server_address
         self.response_status = None
@@ -82,6 +83,48 @@ class Request(object):
         else:
             wsgi_multiprocess = True
             
+            
+
+        
+        # authors should be aware that REMOTE_HOST and REMOTE_ADDR
+        # may not qualify the remote addr:
+        # http://www.ietf.org/rfc/rfc3875
+        try:
+            if 'X-Forwarded-For' in self.parser.headers_dict:
+                forward_adress = self.parser.headers_dict.get('X-Forwarded-For')
+                
+                # we only took the last one
+                # http://en.wikipedia.org/wiki/X-Forwarded-For
+                if "," in forward_adress:
+                    forward_adress = forward_adress.split(",")[-1].strip()
+                
+                if ":" in forward_adress:
+                    remote_addr, remote_port = forward_adress.split(':')
+                else:
+                    remote_addr, remote_port = (forward_adress, '')
+            elif self.client_address is not None:
+                remote_addr, remote_port = self.client_address
+            else:  
+                remote_addr, remote_port = ('127.0.0.1', '')
+        except:
+             remote_addr, remote_port = ('127.0.0.1', '')
+            
+        
+        # Try to server address from headers
+        if 'Host' in self.parser.headers_dict:
+            server_address = self.parser.headers_dict.get('Host')
+        else:
+            server_address = self.server_address
+            
+        if isinstance(server_address, basestring):
+            if ':' in server_address:
+                server_name, server_port = server_address.split(":")
+            else:
+                server_name = server_address
+                server_port = ''
+        else:
+            server_name, server_port = server_address
+        
         environ = {
             "wsgi.url_scheme": 'http',
             "wsgi.input": wsgi_input,
@@ -98,10 +141,10 @@ class Request(object):
             "RAW_URI": self.parser.raw_path,
             "CONTENT_TYPE": self.parser.headers_dict.get('Content-Type', ''),
             "CONTENT_LENGTH": str(wsgi_input.len),
-            "REMOTE_ADDR": self.client_address[0],
-            "REMOTE_PORT": self.client_address[1],
-            "SERVER_NAME": self.server_address[0],
-            "SERVER_PORT": self.server_address[1],
+            "REMOTE_ADDR": remote_addr,
+            "REMOTE_PORT": remote_port,
+            "SERVER_NAME": server_name,
+            "SERVER_PORT": server_port,
             "SERVER_PROTOCOL": self.parser.raw_version
         }
         
@@ -109,7 +152,6 @@ class Request(object):
             key = 'HTTP_' + key.upper().replace('-', '_')
             if key not in ('HTTP_CONTENT_TYPE', 'HTTP_CONTENT_LENGTH'):
                 environ[key] = value
-
         return environ
         
     def start_response(self, status, response_headers, exc_info=None):
